"""
Script Name	      : yhl_171221
Author		      : Yang HeLin
Created		      : 2017/12/19
Last Modified     : 2017/12/19
Version		      : 2
Modifications     :
Description	      : one kind of smoothing technique to remove
                    noise from an image
"""
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

def conv2(img,fil):         
    
    fil_heigh = fil.shape[0]                        #获取卷积核(滤波)的高度
    fil_width = fil.shape[1]                        #获取卷积核(滤波)的宽度
    
    conv_heigh = img.shape[0] - fil_heigh + 1    #确定卷积结果的大小
    conv_width = img.shape[1] - fil_width + 1

    conv = np.zeros((conv_heigh,conv_width),dtype = 'uint8')
    
    for i in range(conv_heigh):
        for j in range(conv_width):                 #逐点相乘并求和得到每一个点
            conv[i][j] = wise_element_sum(img[i:i + fil_heigh,j:j + fil_width ],fil)
    return conv
    
def wise_element_sum(img,fil):          
    return (img * fil).sum()



img=np.array(Image.open('yhl_pic.jpg'))  #打开图像并转化为数字矩阵
rows,cols,cha=img.shape
print(rows,cols,cha)

A=np.ones([3,3])/9
newimg = []
for k in range(3):
    newimg.append(conv2(img[:,:,k],A))

newimg = np.dstack(newimg) 

plt.figure()
plt.imshow(img)
plt.title('original')
plt.axis('off')
plt.figure()
plt.imshow(newimg)
plt.title('Filtered renderings')
plt.axis('off')
plt.show()



'''
Script Name	  : ReadMhd_WriteNifty
Author		  : Boyce_Tian
Created		  : 2018/4/12
Version		  : 1.1
Description	  : a function named ReadMhd_WriteNifty which aims to transform image format from Mhd to Nifty
                input: impath is the path where your pending image is;
                       impath_new is the new path where you want the format changed image is;

'''

import SimpleITK as sitk

def ReadMhd_WriteNifty(impath,impath_new):

    '''
    input:param impath & impsth_new
    return: imsave which aim to save the format changed image to a new designated path;
    all functions used in this function are from SimpleITK
    sitk.ReadImage():aim to read the pending image in
    sitk.GetArrayFromImage():aim to get array from the image
    sitk.GetImageFromArray()&sitk.WriteImage():both achieve to write and save the new image as what we want

    '''

# main part of the function
    image=sitk.ReadImage(impath)
    image_arr=sitk.GetArrayFromImage(image)
    imnew=sitk.GetImageFromArray(image_arr)
    imsave=sitk.WriteImage(imnew,impath_new)
    return imsave

# an example of using the function
# read the new image in the format of Nifity
ReadMhd_WriteNifty('E:\python\PATIENT_DICOM.mhd',
                   'E:\python\PATIENT_DICOM.nii')
imnew_read=sitk.ReadImage('E:\python\PATIENT_DICOM.nii')



from PIL import Image

im_path = r'C:\Users\lenovo\Desktop\11.jpg'
im = Image.open(im_path)
width, height = im.size

print(im.size, width, height)

print(im.format, im.format_description)
 
im.save(r'C:\Users\lenovo\Desktop\11.jpg')
im.show()

import matplotlib.pyplot as plt  
import numpy as np  
  
#To draw y=x^2(-3<=x<=3)  
  
x = np.arange(-3,3.5,0.5)  
y = [ele**2 for ele in x]  
z = [ele *2 for ele in x]  
  
fig = plt.figure(1)  
  
ax = fig.add_subplot(211)  
line1 = ax.plot(x,y,'ro-')  
  
ax = fig.add_subplot(212)  
line2 = ax.plot(x,z,'g-')  
  
plt.show()  


#Python 3.6.2 (v3.6.2:5fd33b5, Jul  8 2017, 04:14:34) [MSC v.1900 32 bit (Intel)] on win32
#Type "copyright", "credits" or "license()" for more information.
L=[1,2,3]
for i in range(len(L)):    #Add one to each number in L
	L[i]+=1            #Or L[i]=L[i]+1

print(L)


"""
Script Name	      : mean_filter
Author		      : Yang HeLin
Created		      : 2017/12/11
Last Modified     : 2017/12/11
Version		      : 1
Modifications     :
Description	      : one kind of smoothing technique to remove
                    noise from an image
"""

#The "window" with 8 neighbors around slides over the image, and
# replacing each entry with the average of neighboring entries.
# param para1: 第一个参数的意义
# param para2: 第二个参数的意义
# param para3: 第三个参数的意义
# rtype:
# return:
# func:
#from skimage import data,io

from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

img=np.array(Image.open('E:\IMAGE-BME\ceshi.jpg'))  #打开图像并转化为数字矩阵
rows,cols,cha=img.shape
print(rows,cols,cha)
newimg=np.array([np.zeros((rows,cols)),np.zeros((rows,cols)),np.zeros((rows,cols))])
print(newimg.shape)
print(newimg)
A=[[1/9,1/9,1/9],[1/9,1/9,1/9],[1/9,1/9,1/9]]
for k in {0,1,2}:
    for i in range(1,rows-1):
        for j in range(1,cols-1):
              a=sum(sum(img[i-1:i+2,j-1:j+2,k]*A))
              newimg[k,i,j]=(a)
             # print(a)
plt.imshow(newimg)
plt.axis('off')
plt.show()



"""
Script Name	      : yhl_171221
Author		      : Yang HeLin
Created		      : 2017/12/19
Last Modified     : 2017/12/19
Version		      : 1
Modifications     :
Description	      : one kind of smoothing technique to remove
                    noise from an image
"""

from PIL import Image
import numpy as np
import matplotlib.pyplot as plt
from skimage import filters

img=np.array(Image.open('yhl_pic.jpg'))  #打开图像并转化为数字矩阵

gaussimg = filters.gaussian_filter(img,sigma=2)

plt.figure()
plt.imshow(gaussimg)
plt.title('gaussian')
plt.axis('off')
plt.figure()
plt.imshow(img)
plt.title(u'original')
plt.axis('off')
plt.show()



import SimpleITK as sitk 
import numpy as np 

'''
Script name   :  itkImageOpenClose
Author        :  Hang Pan
Created       :  2018/05/03
Version       :  1.0
Description   :
	PURPOSE   :  Handle an image(support unsigned 8 integer) with opening or closing filter
	INCLUDING :
	- BinaryMorphologicalOpeningImageFilter
	- BinaryMorphologicalClosingImageFilter
	- OpeningByReconstructionImageFilter
	- ClosingByReconstructionImageFilter
'''


def BinaryMorphologicalOpeningImageFilter(filePath, finalPath):
	'''
	PURPOSE     : Handle an image(support unsigned 8 integer) with BinaryMorphologicalOpeningImageFilter
	INPUTS      : 
	- filePath  : Image file path
	- finalPath : Output image file path
	OUTPUT      : Handled image in the certain final path 
	'''

	itk_img = sitk.ReadImage(filePath, sitk.sitkUInt8)

	img_array = sitk.GetArrayFromImage(itk_img)
	print(img_array)
	print(type(img_array[0][0]))

	Obj = sitk.BinaryMorphologicalOpeningImageFilter()
	output = Obj.Execute(itk_img)
	sitk.WriteImage(output, finalPath)


def BinaryMorphologicalClosingImageFilter(filePath, finalPath):
	'''
	PURPOSE     : Handle an image(support unsigned 8 integer) with BinaryMorphologicalClosingImageFilter
	INPUTS      : 
	- filePath  : Image file path
	- finalPath : Output image file path
	OUTPUT      : Handled image in the certain final path 
	'''

	itk_img = sitk.ReadImage(filePath, sitk.sitkUInt8)

	img_array = sitk.GetArrayFromImage(itk_img)


	Obj = sitk.BinaryMorphologicalClosingImageFilter()
	output = Obj.Execute(itk_img)
	sitk.WriteImage(output, finalPath)

def OpeningByReconstructionImageFilter(filePath, finalPath):
	'''
	PURPOSE     : Handle an image(support unsigned 8 integer) with OpeningByReconstructionImageFilter
	INPUTS      : 
	- filePath  : Image file path
	- finalPath : Output image file path
	OUTPUT      : Handled image in the certain final path 
	'''

	itk_img = sitk.ReadImage(filePath, sitk.sitkUInt8)

	img_array = sitk.GetArrayFromImage(itk_img)
	print(img_array)
	print(type(img_array[0][0]))

	Obj = sitk.OpeningByReconstructionImageFilter()
	output = Obj.Execute(itk_img)
	sitk.WriteImage(output, finalPath)

def ClosingByReconstructionImageFilter(filePath, finalPath):
	'''
	PURPOSE     : Handle an image(support unsigned 8 integer) with ClosingByReconstructionImageFilter
	INPUTS      : 
	- filePath  : Image file path
	- finalPath : Output image file path
	OUTPUT      : Handled image in the certain final path 
	'''

	itk_img = sitk.ReadImage(filePath, sitk.sitkUInt8)

	img_array = sitk.GetArrayFromImage(itk_img)
	print(img_array)
	print(type(img_array[0][0]))

	Obj = sitk.ClosingByReconstructionImageFilter()
	output = Obj.Execute(itk_img)
	sitk.WriteImage(output, finalPath)


BinaryMorphologicalOpeningImageFilter('confirm.jpg','./3.jpg')



"""
  File name: AverageFilter_demo
   Function：实现模板滤波
 Created on: 2017-12-12
     Author: zx
"""

#-*- coding: utf8 -*-

import numpy as np
from PIL import Image
import matplotlib.pyplot as plt
import xlsxwriter
import xlrd


img=Image.open('C:users/zengxiao/Desktop/ImageProcess/lena1.jpg').convert('RGB')    #使用PIL打开图片

radius = 1 ;    #模板的半径

#得到模板
sideLength = radius * 2 + 1
template = np.zeros((sideLength, sideLength))
for i in range(sideLength):
    for j in range(sideLength):
        template[i, j] = 1
all = template.sum()
template = template / all

#对图像进行滤波
arr = np.array(img)
height = arr.shape[0]
width = arr.shape[1]
newData = np.zeros((height, width))
for i in range(radius, height - radius):
    for j in range(radius, width - radius):
        t = arr[i - radius:i + radius + 1, j - radius:j + radius + 1]
        a = np.multiply(t, template)
        newData[i, j] = a.sum()
newImage = Image.fromarray(newData)

#显示滤波后的图像
plt.figure('模板滤波')
plt.imshow(newImage)
plt.show()






#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : itkImageEnhancement
Author		  : ZengXiao
Created		  : 2018/4/20
Version		  : 1.0
Description	  :
  PURPOSE     ：Enhance images with the function of SimpleITK
  INPUTS      :
  - path      : The path of the file which to be converted. Type of data: str
  - my_method : The methods of Image Enhancement. Type of data: str
  - method    :
    - GSD     : Gray scale Dilate
    - AHE     : Adaptive Histogram Equalization
    - LS      : Laplace Sharpening
  OUTPUT      : The enhanced image
  
"""
import SimpleITK as sitk


def itkImageEnhancement(path, my_method):
    # Define function itkImageEnhancement
    img = sitk.ReadImage(path)                  # Call SimpleITK to read the image
    result = []

    # img = sitk.HistogramMatching(img1, img2)  # 直方图匹配 Histogram Matching
    # 函数参数不一样，使用*args 出现error 尚未解决

    if my_method == 'GSD':                      # chose the method of enhancement
        result = sitk.GrayscaleDilate(img)      # save the file named result in the same path
        print('Gray Scale Dilate')              # Tell the user function worked successfully
    elif my_method == 'AHE':
        result = sitk.AdaptiveHistogramEqualization(img)
        print('Adaptive Histogram Equalization')
    elif my_method == 'LS':
        result = sitk.LaplacianSharpening(img)
        print('Laplace Sharpening')
    else:
        print('Error')   # Return an error

    return result


''' Test function '''
path = "D:/PythonCode/zx/src/a.dcm"            # Input path parameters
my_method = "AHE"                              # Method
image = itkImageEnhancement(path, my_method)   # Call function

my_method = "zx"
image = itkImageEnhancement(path, my_method)   # Try incorrect method



"""
Script Name   : itkEdgePreservedSmoothing
Author        : Han
Created       : 2018/5/3
Version       : 1.1
Description   :
  PURPOSE     : This is a function that smooth image with edge preserved
  INPUTS      :
  - im_arr    : Im_arr is the array of the image
                Type of data: ndarray
  - type_str  : the type of edge-preserved smoothing that you want to process
                Type of data: str
                'B'= 'BilateralImageFilter'
                'MC' = 'MinMaxCurvatureFlowImageFilter'
                'CF' = 'CurvatureFlowImageFilter'
                'CAD' = 'CurvatureAnisotropicDiffusionImageFilter'
                'GAD' = 'GradientAnisotropicDiffusionImageFilter'

   OUTPUTS    :
   - im_new   : Image
"""


import SimpleITK as sitk


def itkEdgePreservedSmoothing(im_arr, type_str):

    func_1 = 'B'
    func_2 = 'MC'
    func_3 = 'CF'
    func_4 = 'CAD'
    func_5 = 'GAD'

    # get an image from the array input

    image = sitk.GetImageFromArray(im_arr)
    image = sitk.Cast(image, sitk.sitkFloat32)

    # find out the way to process the image according to type_str
    # smooth the image
    if type_str == func_1:

        im_new = sitk.Bilateral(image)

    elif type_str == func_2:

        im_new = sitk.MinMaxCurvatureFlow(image)

    elif type_str == func_3:

        im_new = sitk.CurvatureFlow(image)

    elif type_str == func_4:

        im_new = sitk.CurvatureAnisotropicDiffusion(image)

    elif type_str == func_5:

        im_new = sitk.GradientAnisotropicDiffusion(image)

    else:
        print('Please check your spelling,'
              'and try again.')
    return im_new


# an example of using the function
if __name__ == '__main__':

    impath = './src_image/CT159.dcm'
    image = sitk.ReadImage(impath)
    print(image.GetDimension(), image.GetPixelID(), image.GetPixelIDValue(), image.GetSize())
    image_arr = sitk.GetArrayFromImage(image)
    image_new = itkEdgePreservedSmoothing(image_arr, 'GAD')
    print(image_new.GetDimension(), image_new.GetPixelID(), image_new.GetPixelIDValue(), image_new.GetSize())
    image_test = sitk.Cast(image_new, sitk.sitkInt16)
    print(image_test.GetDimension(), image_test.GetPixelID(), image_test.GetPixelIDValue(), image_test.GetSize())
    sitk.WriteImage(image_test, './src_image/yi.dcm')



"""
 Script Name   : itkImageSmoothing
 Author		   : Boyce_Tian
 Created       : 2018/4/19
 Version       : 2.0
 Description   :
   PURPOSE     : Serving several ways to smooth the image
   INPUTS      :
   - image_arr : The array of the image you want to smooth.
   - type_str  :
     - DGauss  : Smooth image by function sitk.DiscreteGaussian()
     - Blur    : Smooth image by function sitk.BinomialBlur();
     - RGauss  : Smooth image by function sitk.RecursiveGaussian();
   OUTPUT      :
   - im_new    : An image as the result of smoothing.
"""

import SimpleITK as sitk

def itkImageSmoothing(Im_arr , type_str):

    # define the three ways
    func_1 = 'Discrete Gaussian'
    func_2 = 'Binomial Blurring'
    func_3 = 'Recursive Gaussian'

    #get an image from the array input
    image = sitk.GetImageFromArray(Im_arr)

    # find out the way to process the image according to type_str
    # smooth the image
    if type_str == func_1:

        im_new = sitk.DiscreteGaussian(image)

    elif type_str == func_2:

        im_new = sitk.BinomialBlur(image)

    elif type_str == func_3:

        im_new = sitk.RecursiveGaussian(image)

    else:
        print('Please check your spelling,'
              'and try again.')

    return im_new


# an example of using the function
# smooth an image by the 'Discrete Gaussian' and save the output
if __name__ == '__main__':
    impath = './src_image/CT159.dcm'
    im= sitk.ReadImage(impath)
    image_arr = sitk.GetArrayFromImage(im)
    image_new=itkImageSmoothing(image_arr ,'Discrete Gaussian')
    sitk.WriteImage(image_new,'./src_image/haha.dcm')


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : itkVectorFuzzyConnectednessImageFilter
Author		  : zengxiao
Created		  : 2018/7/11
Version		  : 2.0
Description	  : Sorry, I didn't understand how does the code work, so I just copied genispan's code from CSDN.
                The courses arrangement of this week is too full, and I have not been able to explore it very well.
  PURPOSE     : Vtk display surface
  Modified    : Change to a simple display method.
"""

import vtk

filename = "Liver.stl"

reader = vtk.vtkSTLReader()
reader.SetFileName(filename)

mapper = vtk.vtkPolyDataMapper()

mapper.SetInputConnection(reader.GetOutputPort())

actor = vtk.vtkActor()
actor.SetMapper(mapper)

# Create a rendering window and renderer
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)

# Create a renderwindowinteractor
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Assign actor to the renderer
ren.AddActor(actor)

# Enable user interface interactor
iren.Initialize()
renWin.Render()
iren.Start()


@version: python 3.3.0
@author: Mirabelle1
@lisenses: UTF-8
@see: Baidu
@note: to use the functions of the three libraries
@bug: can not find the path of the 'text.xls'
@attention: null


# param im_path: the path of the image
# param im:  the image
# param width, height: the width, height of the image
# rtype: int
# return: int(0)
# func:  to get the basic information like width, height of the image and save it.

from PIL import Image

im_path = r'C:\Users\lenovo\Desktop\11.jpg'
im = Image.open(im_path)
width, height = im.size

print(im.size, width, height)

print(im.format, im.format_description)
 
im.save(r'C:\Users\lenovo\Desktop\11.jpg')
im.show()


# param f: to build a new workbook
# param sheet1:  to save the data of the image in it
# param l_: the width, height of the image
# rtype: accounting to the list
# return: accounting to the list
# func:  to build a new workbook and  to save the data of the image in it.
import xlwt
f = xlwt.Workbook()
sheet1 = f,add_sheet(u'sheet1',cell_overwrite_=True)
l_=[im.size, width, height,im.format, im.format_description]
for i in range(len(l_)):
    sheet1.write(0,i,i)
f.save('text,xls')


# param x: Abscissa
# param y:  one of the ordinates 
# param z: the other ordinate
# param fig:to figure it
# rtype: float
# return: float
# func: To draw y=x^2(-3<=x<=3) and z=x*2(-3<=x<=3)

import matplotlib.pyplot as plt  
import numpy as np  
x = np.arange(-3,3.5,0.5)  
y = [ele**2 for ele in x]  
z = [ele *2 for ele in x]  
  
fig = plt.figure(1)  
  
ax = fig.add_subplot(211)  
line1 = ax.plot(x,y,'ro-')  
  
ax = fig.add_subplot(212)  
line2 = ax.plot(x,z,'g-')  
  
plt.show()  



'''
Script Name	  : ReadMhd_Write_2_nrry
Author		  : libo_hui
Created		  : 2018/4/13
Version		  : 2.0
Description	  : a function named ReadMhd_Write_2_nrry which aims to transform image format from Mhd to nrry
                input: impath is the path where your pending image is;
                       impath_new is the new path where you want the format changed image is;

'''

import SimpleITK as sitk

def ReadMhd_WriteNifty(impath,impath_new):

    '''
    input:param impath & impsth_new
    return: imsave which aim to save the format changed image to a new designated path;
    all functions used in this function are from SimpleITK
    sitk.ReadImage():aim to read the pending image in
    sitk.GetArrayFromImage():aim to get array from the image
    sitk.GetImageFromArray()&sitk.WriteImage():both achieve to write and save the new image as what we want

    '''

# main part of the function
    #src : target image
    #img_array : the array form image
    #img : the new created image what we want
    #imresult : return value
    src = sitk.ReadImage(impath)
    img_array = sitk.GetArrayFromImage(src)
    img = sitk.GetImageFromArray(img_array)
    imresult = sitk.WriteImage(img, impath_new)

    return imresult

# an example of using the function
# read the new image in the format of nrry
ReadMhd_WriteNifty('/Users/huilibo/PycharmProjects/test1/PATIENT_DICOM.mhd',
                   '/Users/huilibo/PycharmProjects/test1/Result_1.nrrd')
imnew_read=sitk.ReadImage('/Users/huilibo/PycharmProjects/test1/Result_1.nrrd')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : itk3DImageRegistration
Author		  : ZengXiao
Created		  : 2018/5/29
Version		  : 2.0
Description	  :
  PURPOSE     :
  INPUTS      : 3-D image registration
  - fix       : Path of fixed image
                Type of data: str
  - move      : Path of moving image
                Type of data: str
  - transform : Spatial Transform
                Type of data: str

                "euler": rigid transform
                "similarity": similarity transform
                "affine": affine transform
                "nonlinear": affine transform

  - metric    : Similarity Metric
                Type of data: str

                "information": Mutual Information
                "ANTS": Unknown
                "correlation": Normalized Cross Correlation
                "Hinformation": Joint Histogram Mutual Information
                "MeanSquares": Mean Squared Difference

  - interpolator: Optimizer
                  Type of data: str

                  "linear": Linear Interpolation
                  "nearest": Nearest Neighbor Interpolation
                  "BSpline": B-Spline Interpolation

  - optimizer  : Substitution value
                 Type of data: int

                 "AsGradient": Gradient Descendant
                 "StepGradient": Conjugate Gradient Descendant
                 "LBFGSB": LBFGSB
                 "LBFGS2": LBFGS2
                 "Exhaustive": Exhaustive
                 "Amoeba": Amoeba
                 "Weights": Weights
                 "Powell": Powell

   OUTPUTS    : None

"""
import SimpleITK as sitk
import os


def itk3DImageRegistration(fix, move, transform, metric, interpolator, optimizer):
    switch_SetTransform = {
        "euler": sitk.Euler3DTransform(fixed_image.GetDimension()),
        "similarity": sitk.Similarity3DTransform(fixed_image.GetDimension()),
        "affine": sitk.AffineTransform(fixed_image.GetDimension()),
        "nonlinear": sitk.BSplineTransform(fixed_image.GetDimension()),
    }
    switch_metric = {
        "information": registration_method.SetMetricAsMattesMutualInformation(numberOfHistogramBins=50),
        "ANTS": registration_method.SetMetricAsANTSNeighborhoodCorrelation(radius=50),
        "correlation": registration_method.SetMetricAsCorrelation(),
        "Hinformation": registration_method.SetMetricAsJointHistogramMutualInformation(numberOfHistogramBins=20,
                                                                                       varianceForJointPDFSmoothing=1.5),
        "MeanSquares": registration_method.SetMetricAsMeanSquares(),
    }
    switch_interpolator = {
        "linear": sitk.sitkLinear,
        "nearest": sitk.sitkNearestNeighbor,
        "BSpline": sitk.sitkBSpline,
    }
    switch_optimizer = {
        "AsGradient": registration_method.SetOptimizerAsGradientDescent(learningRate=1.0, numberOfIterations=100,
                                                                        convergenceMinimumValue=1e-6,
                                                                        convergenceWindowSize=10),
        "StepGradient": registration_method.SetOptimizerAsRegularStepGradientDescent(relaxationFactor=0.5,
                                                                                     gradientMagnitudeTolerance=1e-4,
                                                                                     maximumStepSizeInPhysicalUnits=0.0),
        "LBFGSB": registration_method.SetOptimizerAsLBFGSB(gradientConvergenceTolerance=1e-5, numberOfIterations=500,
                                                           maximumNumberOfCorrections=5,
                                                           maximumNumberOfFunctionEvaluations=2000,
                                                           maximumStepSizeInPhysicalUnits=0.0),
        "LBFGS2": registration_method.SetOptimizerAsLBFGS2(numberOfIterations=0, hessianApproximateAccuracy=6,
                                                           deltaConvergenceDistance=0, deltaConvergenceTolerance=1e-5,
                                                           lineSearchMaximumEvaluations=40, lineSearchMinimumStep=1e-20,
                                                           lineSearchMaximumStep=1e20, lineSearchAccuracy=1e-4),
        "Exhaustive": registration_method.SetOptimizerAsExhaustive(stepLength=1.0),
        "Amoeba": registration_method.SetOptimizerAsAmoeba(parametersConvergenceTolerance=1e-8,
                                                           functionConvergenceTolerance=1e-4),
        "Weights": registration_method.SetOptimizerWeights(),
        "Powell": registration_method.SetOptimizerAsPowell(numberOfIterations=100, maximumLineIterations=100,
                                                           stepLength=1,
                                                           stepTolerance=1e-6, valueTolerance=1e-6),
    }
    # read the images and casting the pixel type to Float32 (or Float64)
    fixed_image = sitk.ReadImage(fix, sitk.sitkFloat32)
    moving_image = sitk.ReadImage(move, sitk.sitkFloat32)

    # align the centers of the two volumes and set the center of rotation to the center of the fixed image.
    initial_transform = sitk.CenteredTransformInitializer(fixed_image,
                                                          moving_image,
                                                          switch_SetTransform.get(transform, 'Incorrect parameter input'),
                                                          sitk.CenteredTransformInitializerFilter.GEOMETRY)
    moving_resampled = sitk.Resample(moving_image, fixed_image, initial_transform,
                                     sitk.sitkLinear, 0.0, moving_image.GetPixelID())
    registration_method = sitk.ImageRegistrationMethod()

    # Similarity metric settings.
    switch_metric.get(metric, 'Incorrect parameter input'),
    registration_method.SetMetricSamplingStrategy(registration_method.RANDOM)
    registration_method.SetMetricSamplingPercentage(0.01)

    # Interpolator settings.
    registration_method.SetInterpolator(switch_interpolator.get(interpolator, 'Incorrect parameter input'))

    # Optimizer settings.
    switch_optimizer.get(optimizer,'Incorrect parameter input')

    registration_method.SetOptimizerScalesFromPhysicalShift()

    registration_method.SetOptimizerScalesFromPhysicalShift(centralRegionRadius=5,
                                                            smallParameterVariation=0.01)
    # Setup for the multi-resolution framework.
    registration_method.SetShrinkFactorsPerLevel(shrinkFactors=[4, 2, 1])
    registration_method.SetSmoothingSigmasPerLevel(smoothingSigmas=[2, 1, 0])
    registration_method.SmoothingSigmasAreSpecifiedInPhysicalUnitsOn()

    # Don't optimize in-place, we would possibly like to run this cell multiple times.
    registration_method.SetInitialTransform(initial_transform, inPlace=False)

    final_transform = registration_method.Execute(sitk.Cast(fixed_image, sitk.sitkFloat32),
                                                  sitk.Cast(moving_image, sitk.sitkFloat32))

    # Query the registration method to see the metric value and the reason the optimization terminated.
    print('Final metric value: {0}'.format(registration_method.GetMetricValue()))
    print('Optimizer\'s stopping condition, {0}'.format(registration_method.GetOptimizerStopConditionDescription()))
    sitk.WriteImage(moving_resampled, os.path.join('Output', 'D:\PythonCode\zx\src\moving.mhd'))



"""
Script Name	  : itkImageLevelSetSegmentation
Author		  :
Created		  : 2018/5/3
Version		  : 1.2
Description   :
  PURPOSE     : Processing image using level-set segmentation
  INPUTS      :
  - Im_arr    : Im_arr is the array of the image
                Type of data: ndarray
  - type_str  : the type of image level-set segmentation that you want to process
                Type of data: str
                'SD'  :  'ShapeDetectionLevelSet'
                'GAC' ： 'GeodesicActiveContourLevelSet'
                'TH'  ： 'ThresholdSegmentationLevelSet'
                'FA'  ： 'FastMarching'
                'FAB' ： 'FastMarchingBase'
                'LP'  ： 'LaplacianSegmentationLevelSet'
   OUTPUTS    :
   - im_new   : Image
"""

import SimpleITK as sitk
import itkRegionGrow


def itkImageLevelSetSegmentation(Im_arr, type_str, seedlist=0):

    # define the six way
    func_1 = 'SD'
    func_2 = 'GAC'
    func_3 = 'TH'
    func_4 = 'FA'
    func_5 = 'FAB'
    func_6 = 'LP'

    # get an image from the array input
    image = sitk.GetImageFromArray(Im_arr)

    # find out the way to process the image according to type_str
    # smooth the image
    if type_str == func_1:
        image = sitk.Cast(image,sitk.sitkFloat32)
        image_pre=sitk.CannyEdgeDetection(image)
        im_new = sitk.ShapeDetectionLevelSet(image,image_pre)

    elif type_str == func_2:
        #image = sitk.GradientMagnitudeRecursiveGaussian(image)
        #featureImage = sitk.BoundedReciprocal(image)
        #seg = sitk.Image(image.GetSize(),sitk.sitkUInt8)
        #seg.CopyInformation(image)
        #seg[seedlist] = 1
        #distance = sitk.SignedMaurerDistanceMap(seg, insideIsPositive=True, useImageSpacing=True)
        #init_ls = sitk.BinaryThreshold(distance, -1000, 10)
        #init_ls = sitk.Cast(init_ls, featureImage.GetPixelID())*-1+0.5
        seg = itkRegionGrow.itkRegionGrow(Im_arr, 'CC', seedlist)
        init_ls = sitk.SignedMaurerDistanceMap(seg, insideIsPositive=True, useImageSpacing=True)
        im_new = sitk.GeodesicActiveContourLevelSet(init_ls, sitk.Cast(image, sitk.sitkFloat32))

    elif type_str == func_3:
        #seg = sitk.Image(image.GetSize(),sitk.sitkUInt8)
        #seg.CopyInformation(image)
        #seg[seedlist] = 1
        #seg = sitk.BinaryDilate(seg, 10)
        seg = itkRegionGrow.itkRegionGrow(Im_arr, 'CC', seedlist)
        stats = sitk.LabelStatisticsImageFilter()
        stats.Execute(image, seg)
        lower_threshold = stats.GetMean(1)-1.5*stats.GetSigma(1)
        upper_threshold = stats.GetMean(1)+1.5*stats.GetSigma(1)
        init_ls = sitk.SignedMaurerDistanceMap(seg, insideIsPositive=True, useImageSpacing=True)
        im_new = sitk.ThresholdSegmentationLevelSet(init_ls,sitk.Cast(image,sitk.sitkFloat32), lower_threshold, upper_threshold)

    elif type_str == func_4:
        im_new = sitk.FastMarching(image, seedlist)

    elif type_str == func_5:
        im_new = sitk.FastMarchingBase(image, seedlist)

    elif type_str == func_6:
        #seg = sitk.Image(image.GetSize(), sitk.sitkUInt8)
        #seg.CopyInformation(image)
        #seg[index] = 1
        #seg = sitk.BinaryDilate(seg, 8)
        seg = itkRegionGrow.itkRegionGrow(Im_arr, 'CC', seedlist)
        init_ls = sitk.SignedMaurerDistanceMap(seg, insideIsPositive=True, useImageSpacing=True)
        im_new = sitk.LaplacianSegmentationLevelSet(init_ls, sitk.Cast(image, sitk.sitkFloat32))

    else:

        print('Please check your spelling,'
              'and try again.')

    return im_new


# an example of using the function
if __name__ == '__main__':
    impath = './src_image/Result.nrrd'
    im = sitk.ReadImage(impath)
    seedlist = [[259, 158, 116]]
    image_arr = sitk.GetArrayFromImage(im)
    image_new = itkImageLevelSetSegmentation(image_arr, 'LP', seedlist)
    print(image_new.GetSize(), image_new.GetDimension(), image_new.GetPixelID())
    sitk.Show(image_new)
    image_out = sitk.Cast(image_new,sitk.sitkInt16)
    sitk.WriteImage(image_out, './src_image/hii.nrrd')


"""
Script Name	      : mean_filter
Author		      : Yang HeLin
Created		      : 2017/12/11
Last Modified     : 2017/12/11
Version		      : 1
Modifications     :
Description	      : one kind of smoothing technique to remove
                    noise from an image
"""

#The "window" with 8 neighbors around slides over the image, and
# replacing each entry with the average of neighboring entries.
# param para1: 第一个参数的意义
# param para2: 第二个参数的意义
# param para3: 第三个参数的意义
# rtype:
# return:
# func:
#from skimage import data,io

from PIL import Image


img=Image.open('E:\IMAGE-BME\ceshi.jpg') #打开图像并转化为数字矩阵
pix = img.load()
width = img.size[0]
height = img.size[1]

for i in range(1,width-1):
    for j in range(1, height-1):
        r1,g1,b1=pix[i-1,j-1]
        r2,g2,b2=pix[i-1,j]
        r3,g3,b3=pix[i-1,j+1]
        r4,g4,b4=pix[i,j-1]
        r5,g5,b5=pix[i,j]
        r6,g6,b6=pix[i,j+1]
        r7,g7,b7=pix[i+1,j-1]
        r8,g8,b8=pix[i+1,j]
        r9,g9,b9=pix[i+1,j+1]
        img.putpixel([i,j],((r1+r2+r3+r4+r5+r6+r7+r8+r9)/9,(g1+g2+g3+g4+g5+g6+g7+g8+g9)/9,(b1+b2+b3+b4+b5+b6+b7+b8+b9)/9))

img.show()
img.save("c.jpg")


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : itkVectorFuzzyConnectednessImageFilter
Author		  : ZengXiao
Created		  : 2018/5/3
Version		  : 1.0
Description	  :
  PURPOSE     : I cannot find any Fuzzy Connectedness content in SimpleITK. I found function VectorConfidenceConnected
                which I think have the similar effects.
  INPUTS      :
  - image     : The path of image
                Type of data: str
  - seedList  : Seed
                Type of data: list
  - numberOFIterations: Number of iterations
                        Type of data: int
  - multiplier: Multiplier
                Type of data: float
  - initialNeighborhoodRadius: Initial neighborhood radius
                               Type of data: int
  - replaceValue: Substitution value
                  Type of data: int

   OUTPUTS    :
   - image    : Image


"""
import SimpleITK as sitk


def itkVectorFuzzyConnectednessImageFilter(path, seedList, iterations=4, multiplier=4.5, radius=1, replaceValue=1):
    img = sitk.GetArrayFromImage(sitk.ReadImage(path))
    result = sitk.VectorConfidenceConnected(img, seedList, iterations, multiplier, radius, replaceValue)
    return result




"""
Script Name :ph_sitk.py
Author :ph
Created :a simple example of sitk
Version :python 3.6.0

"""
import SimpleITK as sitk
 
# Create an image
pixelType = sitk.sitkUInt8
imageSize = [128, 128]
image     = sitk.Image( imageSize, pixelType )
 

# Create a face image
faceSize   = [64, 64]
faceCenter = [64, 64]
face       = sitk.GaussianSource(pixelType, imageSize, faceSize, faceCenter)
 
# Create eye images
eyeSize    = [5, 5]
eye1Center = [48, 48]
eye2Center = [80, 48]
eye1       = sitk.GaussianSource(pixelType, imageSize, eyeSize, eye1Center, 150)
eye2       = sitk.GaussianSource(pixelType, imageSize, eyeSize, eye2Center, 150)
 
# Apply the eyes to the face
face = face - eye1 - eye2
face = sitk.BinaryThreshold(face, 200, 255, 255)
 
 # Create the mouth
mouthRadii  = [30, 20]
mouthCenter = [64, 76]
mouth       = 255 - sitk.BinaryThreshold( sitk.GaussianSource(pixelType, imageSize, mouthRadii, mouthCenter), 200, 255, 255 )

# Paste the mouth into the face
mouthSize = [64, 18]
mouthLoc  = [32, 76]
face      = sitk.Paste(face, mouth, mouthSize, mouthLoc, mouthLoc)
 
# Apply the face to the original image
image = image+face
 
# Display the results
sitk.Show( image, title="Hello World: Python", debugOn=True )


from __future__ import print_function
import SimpleITK as sitk
import sys
import os
if len ( sys.argv ) < 4:
    print( "Usage: %s <input> <sigma> <output>" % ( sys.argv[0] ) )
    sys.exit ( 1 )
image = sitk.ReadImage( sys.argv[1] )
pixelID = image.GetPixelID()
image  = sitk.SmoothingRecursiveGaussian( image,  float( sys.argv[2] ) )
sitk.WriteImage( sitk.Cast( image, pixelID ), sys.argv[3] )



#@title     ： homework2_ECC1
#@update by : Yang Helin 20171111
#@modify    : show in a standard format



#application_Pillow
from PIL import Image

  im = Image.open('ceshi.JPG','JPRG')
    print(im.format, im.size, im.mode)
  im.thumbnail((200, 100))
  im.save('suoxiao.jpg','JPEG')


#application_Numpy,Matplotlib
#plot function and add legend
#try to find similarities and differences with matlab
import numpy             as np
import matplotlib.pyplot as plt

  x = np.linspace(0, 10, 1000)
  y = np.sin(x)
  z = np.cos(x**2)
    plt.figure(figsize=(8,4))
    plt.plot(x,y,label="sin(x)",color="red",linewidth=2)
    plt.plot(x,z,"b--",label="cos(x^2)")
    plt.xlabel("Time(s)")
    plt.ylabel("Volt")
    plt.title("First Example")
    plt.ylim(-1.2,1.2)
    plt.legend()
    plt.show()


import numpy as np
from scipy.fftpack import fft,ifft
import matplotlib.pyplot as plt



#采样点选择1400个，因为设置的信号频率分量最高为600赫兹，根据采样定理知采样频率要大于信号频率2倍，所以这里设置采样频率为1400赫兹（即一秒内有1400个采样点，一样意思的）
x=np.linspace(0,1,1400)

#设置需要采样的信号，频率分量有180，390和600
y=7*np.sin(2*np.pi*180*x) + 2.8*np.sin(2*np.pi*390*x)+5.1*np.sin(2*np.pi*600*x)

yy=fft(y)                     #快速傅里叶变换
yreal = yy.real               # 获取实数部分
yimag = yy.imag               # 获取虚数部分

yf=abs(fft(y))                # 取绝对值
yf1=abs(fft(y))/len(x)           #归一化处理
yf2 = yf1[range(int(len(x)/2))]  #由于对称性，只取一半区间

xf = np.arange(len(y))        # 频率
xf1 = xf
xf2 = xf[range(int(len(x)/2))]  #取一半区间


plt.subplot(221)
plt.plot(x[0:50],y[0:50])
plt.title('Original wave')

plt.subplot(222)
plt.plot(xf,yf,'r')
plt.title('FFT of Mixed wave(two sides frequency range)',fontsize=7,color='#7A378B')  #注意这里的颜色可以查询颜色代码表

plt.subplot(223)
plt.plot(xf1,yf1,'g')
plt.title('FFT of Mixed wave(normalization)',fontsize=9,color='r')

plt.subplot(224)
plt.plot(xf2,yf2,'b')
plt.title('FFT of Mixed wave)',fontsize=10,color='#F08080')


plt.show()





"""
 Script Name   : itkImageEdgeFiltering
 Author		   : Helin Yang
 Created       : 2018/4/19
 Version       : 3.0
 Description   :
   PURPOSE     : Compute the edge of the image 
   INPUTS      :
   - image_arr : The array of the image which to be converted.
   - method    :
     - Gass    : Computes the Magnitude of the Gradient of an image by convolution with the first derivative of a Gaussian.
     - Grad    : Computes the gradient magnitude of an image region at each pixel with a simple finite difference method.
   OUTPUT      : The edge of the image
"""
import SimpleITK as sitk

def itkImageEdgeFiltering(image_arr,method='Grad'):

    if method == 'Gass':
        sitk.GradientMagnitudeRecursiveGaussian(image_arr)
        print('Gradient Magnitude Recursive Gaussian')
    elif method == 'Grad':
        sitk.GradientMagnitude(image_arr)
        print('Gradient Magnitude')
    else:
        print('Error')



# An example
# Please don't forget to import those modules as follows
"""
import itkImageEdgeFiltering
import SimpleITK as sitk
image = sitk.ReadImage('E:\IMAGE-BME\yanghelin\PATIENT_DICOM.mhd')
image_new = itkImageEdgeFiltering.itkImageEdgeFiltering (image,'Gass')
sitk.WriteImage( image_new ,.\PATIENT_DICOM_new.mhd)
"""



"""
Script Name	  : Watershed_example.py
Author		  : Han
Created		  : 2017/12/25
Version		  : 1.0
Description	  : actually，i don't know how it works
"""
import SimpleITK as sitk
import matplotlib.pyplot as plt


# param src        : the images that you want to plot
# description      : this is a function which can show the output image using matplotlib
def my_show(*args):
    for count, src in enumerate(args):
        nda = sitk.GetArrayViewFromImage(src)
        plt.subplot(1, len(args), count + 1)
        plt.imshow(nda)
    plt.show()


img = sitk.ReadImage('CT159.dcm', sitk.sitkFloat32)
img = img[:, :, 0]

feature_image = sitk.GradientMagnitude(img)
watershed_img = sitk.MorphologicalWatershed(feature_image,  markWatershedLine=True,
                                            fullyConnected=False, level=100)
my_show(img, feature_image, sitk.LabelToRGB(watershed_img))



from PIL import Image
im = Image.open('ceshi.JPG')
print(im.format, im.size, im.mode)
im.thumbnail((200, 100))
im.save('suoxiao.jpg', 'JPEG')
import numpy as np
import matplotlib.pyplot as plt
x = np.linspace(0, 10, 1000)
y = np.sin(x)
z = np.cos(x**2)
plt.figure(figsize=(8,4))
plt.plot(x,y,label="sin(x)",color="red",linewidth=2)
plt.plot(x,z,"b--",label="cos(x^2)")
plt.xlabel("Time(s)")
plt.ylabel("Volt")
plt.title("First Example")
plt.ylim(-1.2,1.2)
plt.legend()
plt.show()



"""
 Script Name	  : meshSubdivision
 Author		  : BoyceTian
 Created		  : 2018/7/20
 Version		  : 1.0
 Description	  :
   PURPOSE     : An example for subdividing meshes by using pymesh
   INPUTS      :
   - filepath  : Path of original mesh
                 Type of data: .stl
   - filenew   : Path of subdivided mesh
                 Type of data: stl
   - mesh      : original mesh
                 Type of data: str
   - mesh_new  : subdivided mesh

    OUTPUTS    :
"""

import pymesh

# get the path of the original mesh and set the path of the Subdivided mesh
filepath="./Spleen_3D-interpolation.stl"
filenew="./Spleen_3D-interpolation_new.stl"
#get the original mesh
mesh=pymesh.load_mesh(filepath)
#subdivide the mesh
mesh_new=pymesh.subdivide(mesh,order=3,method='simple')
#write the subdivided mesh as a new file
pymesh.save_mesh(filenew,mesh_new)



# -*- coding: utf-8 -*-

from PIL import Image

from pylab import *

from numpy import *

from scipy.ndimage import filters

#读取图片,灰度化并转为数组
im = array(Image.open('./test.jpg').convert('L'))

#进行σ = 2的高斯滤波
im2 = filters.gaussian_filter(im,2)

#进行σ = 5的高斯滤波
im3 = filters.gaussian_filter(im,5)

#进行σ = 10的高斯滤波
im4 = filters.gaussian_filter(im,10)

#显示图像
gray()

subplot(141)

title('source')

imshow(im)

subplot(142)

title('sigma=2')

imshow(im2)

subplot(143)

title('sigma=5')

imshow(im3)

subplot(144)

title('sigma=10')

imshow(im4)

show()




#title      :Tianzhihao_work3
#author     :Tianzhihao
#created    :20171108
#modified   :20171116
#description:draw a colored scatter by using python

import matplotlib.pyplot as plt
import numpy as ny
from PIL import Image

#生成随机数
a=ny.random.randint(55,1090,(100))
b=list(range(100))

#绘制随机数图像
plt.scatter(b,a,c=a,cmap=plt.cm.Reds,edgecolor='none',s=50)
plt.title("colored random number",fontsize=24)
plt.xlabel("random num",fontsize=14)
plt.ylabel("num",fontsize=14)
plt.show()
plt.savefig('randomnum.png',bbox_inches='tight')

#读取图像信息
im=Image.open("randomnum.png")
print(im.format,im.size,im.mode)



'''
 Script Name: ReadMhd_Write2Nrrd
 Author		: Helin Yang
 Created    : 2018/4/13
 Version    : 2
 Description:   This function is to convert only one image from the Mhd format to the Nrrd format.
                Given the full address of the image that needs to be converted as well as the
              full address of the new image.
                If run successfully, it prints out a completion statement to reminder you.
 '''

import SimpleITK as sitk


def ReadMhd_Write2Nrrd( mhd_full_path_name,nrrd_full_path_name):
   Image = sitk.ReadImage(mhd_full_path_name)
   image_array = sitk.GetArrayFromImage(Image)
   img = sitk.GetImageFromArray(image_array)
   sitk.WriteImage(img, nrrd_full_path_name)
   print ('The picture has been transferred.')

ReadMhd_Write2Nrrd('E:\IMAGE-BME\yanghelin\PATIENT_DICOM.mhd','E:\IMAGE-BME\yanghelin\PATIENT_DICOM.nrrd')


"""
Script Name	  : Gaussian_example.py
Author		  : Han
Created		  : 2017/12/12
Version		  : 1.0
Description	  : gaussian filter
"""
import math
import numpy as np
from PIL import Image

# param input_image: the image that you want to process
# param size       : the size of gaussian kernel
# param sigma      : a param to get gassian kernel
# return           : the image which has been processed
def gaussian_filter(input_image, size, sigma):
    src = np.array(input_image)
    gaussian_kernel = _get_gaussiankernel(size, sigma)

    border = size // 2
    rows = src.shape[0] - 2 * border
    cols = src.shape[1] - 2 * border
    axis = len(src.shape)
    dst = src.copy()

    if axis == 2:
        for i in range(border, rows):
            for j in range(border, cols):
                k = 0
                for a in range(-border, border + 1):
                    for b in range(-border, border + 1):
                        k = k + gaussian_kernel[border + a, border
                                                + b] * src[i + a, j + b]
                dst[i, j] = k
        output_image = Image.fromarray(dst)
        return output_image

    if axis == 3:
        for i in range(border, rows):
            for j in range(border, cols):
                k = np.zeros(3)
                for a in range(-border, border + 1):
                    for b in range(-border, border + 1):
                        k = k + gaussian_kernel[border + a, border
                                                + b] * src[i + a, j + b, :]
                    a = a + 1
                dst[i, j, :] = k[:]
        output_image = Image.fromarray(dst)
        return output_image

# To get the gassian kernel
def _get_gaussiankernel(size, sigma):
    kernel = np.zeros((size, size))
    origin = size // 2
    sum = 0

    for i in range(size):
        x2 = (i - origin)**2
        for j in range(size):
            y2 = (j - origin)**2
            g = math.exp(-(x2 + y2) / (2 * sigma * sigma))
            sum = sum + g
            kernel[i, j] = g
    kernel = kernel / sum
    return kernel


if __name__ == '__main__':

    pic_src = Image.open('1.jpg')
    pic_dst = gaussian_filter(pic_src, 3, 1)
    pic_dst.save('hello.jpg')



"""
Script Name   :zx_homework_1228
Author        :zx
Created on    :2017/12/28
Last Modified :2017/12/28
Version       :1.0
Modifications :zx
Description   :Demons图像变形配准
"""

from __future__ import print_function
import SimpleITK as sitk

def command_iteration(filter):
    print("{0:3} = {1:10.5f}".format(filter.GetElapsedIterations(),
                                     filter.GetMetric()))

fixed = sitk.ReadImage('C:users/zengxiao/Desktop/ImageProcess/lena1.jpg', sitk.sitkFloat32)

moving = sitk.ReadImage('C:users/zengxiao/Desktop/ImageProcess/lena10.jpg', sitk.sitkFloat32)

matcher = sitk.HistogramMatchingImageFilter()
matcher.SetNumberOfHistogramLevels(1024)
matcher.SetNumberOfMatchPoints(7)
matcher.ThresholdAtMeanIntensityOn()
moving = matcher.Execute(moving, fixed)

demons = sitk.DemonsRegistrationFilter()
demons.SetNumberOfIterations(50)
demons.SetStandardDeviations(1.0)

demons.AddCommand(sitk.sitkIterationEvent, lambda: command_iteration(demons))

displacementField = demons.Execute(fixed, moving)

print("-------")
print("Number Of Iterations: {0}".format(demons.GetElapsedIterations()))
print(" RMS: {0}".format(demons.GetRMSChange()))

outTx = sitk.DisplacementFieldTransform(displacementField)

#sitk.WriteTransform(outTx,'C:users/zengxiao/Desktop/ImageProcess/lenaDemons.jpg')

resampler = sitk.ResampleImageFilter()
resampler.SetReferenceImage(fixed);
resampler.SetInterpolator(sitk.sitkLinear)
resampler.SetDefaultPixelValue(100)
resampler.SetTransform(outTx)

out = resampler.Execute(moving)
simg1 = sitk.Cast(sitk.RescaleIntensity(fixed), sitk.sitkUInt8)
simg2 = sitk.Cast(sitk.RescaleIntensity(out), sitk.sitkUInt8)

cimg = sitk.Compose(simg1, simg2, simg1//2.+simg2//2.)



"""
Script Name: ph_filter.py
Author: Panhang
Created: Build a template for filtering
Vertion: Python 3.6
Description: To build a template for filtering
"""

#param im: the array transformed by the image that you want to process
#param sigma: the para to get gaussian kernel
#return: the processed image


from PIL import Image
from pylab import *
from numpy import *
from scipy.ndimage import filters

#read the image and transform it into an array.
im = array(Image.open('C:/Users/lenovo/Desktop/test.jpg').convert('L'))

#To do gaussian filtering whichσ = 2.
im2 = filters.gaussian_filter(im,2)

#To do gaussian filtering whichσ = 5.
im3 = filters.gaussian_filter(im,5)

#To do gaussian filtering whichσ = 10.
im4 = filters.gaussian_filter(im,10)

gray()

#show the image
subplot(141)
title('source')
imshow(im)

subplot(142)
title('sigma=2')
imshow(im2)

subplot(143)
title('sigma=5')
imshow(im3)

subplot(144)
title('sigma=10')
imshow(im4)

show()         



#应用三个库中函数的简单程序
#An example of using library functions
#田志浩 20171108

import matplotlib.pyplot as plt
import numpy as ny
a=ny.random.randint(55,1090,(100))
b=list(range(100))

plt.scatter(b,a,c=a,cmap=plt.cm.Reds,edgecolor='none',s=50)
plt.title("colored random number",fontsize=24)
plt.xlabel("random num",fontsize=14)
plt.ylabel("num",fontsize=14)
plt.show()
plt.savefig('randomnum.png',bbox_inches='tight')

from PIL import Image
im=Image.open("randomnum.png")
from __future__ import print_function
print(im.format,im.size,im.mode)



import SimpleITK as sitk 
import numpy as np 


def ConvertToFormat(filePath, finalPath):
'''
;PURPOSE:
;	Definite a function that can input a img file and convert to an other curtain format img file.
;
;INPUT:
;	A img file path:     (*.mhd)
;	Other format path:   (*.nrrd, *nii)
;
;OUTPUT:
;	Other format in the input path.
;
;RETURN VALUE:
;	void
;
;AUTHER:
;	Hang Pan, Dalian University of Technology
;
;CREATION DATE:
;	2018-04-06
'''
	try:
		img = sitk.GetImageFromArray(sitk.GetArrayFromImage(sitk.ReadImage(filePath)))
	except Exception as e:
		print("ReadError: This file cannot be read!")

	try:
		sitk.WriteImage(img, finalPath)
	except Exception as e:
		print("FormatError: This format is not Support!")




if __name__ == '__main__':
	ConvertToFormat('PATIENT_DICOM.mhd', 'aaaaa.nrrd')




"""
Script Name	  :im_filter
Author		  :Boyce_Tian
Created		  :20171213
Last Modified :
Version		  :1.0
Modifications :
Description	  :Implementation of template filtering
"""


from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

Im=Image.open('E:\\python\\work4.jpg').convert('RGB')
arr_Im=np.array(Im)
width=arr_Im.shape[1]
height=arr_Im.shape[0]
#import the picture and turn it to a matrix

w_tem=3   #the width of template
for i in range(2,width-1):
    for j in range(2,height-1):
        arr=arr_Im[i-1:i+1,j-1:j+1]
        sumarr=np.sum(arr)
        arr_Im[i,j]=sumarr/(w_tem*w_tem)
output_Im=Image.fromarray(arr_Im)
#filtering processing

plt.imshow(output_Im)
plt.show()



"""
Script Name   :GaussianFilter_demo
Author        :zx
Created on    :2017/12/20
Last Modified :2017/12/20
Version       :1.0
Modifications :zx
Description   :调用SimpleITK实现Gaussian滤波
"""

import SimpleITK as sitk

#运用SimpleITK的函数读取图像
reader = sitk.ImageFileReader()
reader.SetFileName('C:users/zengxiao/Desktop/ImageProcess/lena1.jpg')
image = reader.Execute()

pixelID = image.GetPixelID()

gaussian = sitk.SmoothingRecursiveGaussianImageFilter()
gaussian.SetSigma(float(10))         #设置sigma值
image = gaussian.Execute(image)

caster = sitk.CastImageFilter()
caster.SetOutputPixelType(pixelID)
image = caster.Execute(image)

#存储高斯滤波后的图像
writer = sitk.ImageFileWriter()
writer.SetFileName('C:users/zengxiao/Desktop/ImageProcess/lena_Gaussian.jpg')
writer.Execute(image)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : ReadMhd_WriteNifty.py
Author		  : Han
Created		  : 2018/3/31
Version		  : 1.0
Description	  : ReadMhd_WriteNifty
"""
import SimpleITK as sitk

src = sitk.ReadImage("src_image/PATIENT_DICOM.mhd")
img_array = sitk.GetArrayFromImage(src)
img = sitk.GetImageFromArray(img_array)
sitk.WriteImage(img, 'src_image/Result.nii')



"""
Script Name	  : itk2DImageRegistration
Author		  :
Created		  : 2018/5/31
Version		  : 1.0
Description   :
  PURPOSE     :
  INPUTS      :
  - Im_arr1   : fixed_image
                Type of data: not ndarray
  - Im_arr2   : moving_image
                Type of data: not ndarray
  - trans_type         : Spatial Transform algorithms
                        'Euler'       :  'Euler2DTransform'
                        'BSpline'     :  'BSplineTransformInitializer'
                        'Similarity'  :  'Similarity2DTransform'
                        'Affine'      :  'AffineTransform'
  - metric_type        : Similarity Metric algorithms
                        'mutual'      :  'SetMetricAsMattesMutualInformation'
                        'mean'        :  'SetMetricAsMeanSquares'
                        'correlation' :  'SetMetricAsCorrelation'
  - interpolator_type  : Interpolator algorithms
                        'Linear'      :  'sitkLinear'
                        'Neighbor'    :  'sitkNearestNeighbor'
                        'BSpline'     :  'sitkBSpline'
  - optimizer_type     : Optimizer algorithms
                        'GradientLine':  'SetOptimizerAsConjugateGradientLineSearch'
                        'LBFGSB'      :  'SetOptimizerAsLBFGSB'
                        'Powell'      :  'SetOptimizerAsPowell'
                        'Gradient'    :  'SetOptimizerAsGradientDescent'
   OUTPUTS    :
   - im_new   : Image

"""

import SimpleITK as sitk

def itk2DImageRegistration(Im_arr1,Im_arr2,trans_type,metric_type,interpolator_type,optimizer_type):
    # define the transform way
    tfunc_1 = 'Euler'
    tfunc_2 = 'BSpline'
    tfunc_3 = 'Similarity'
    tfunc_4 = 'Affine'

    mfunc1 = 'mutual'
    mfunc2 = 'mean'
    mfunc3 = 'correlation'

    ifunc1 = 'Linear'
    ifunc2 = 'Neighbor'
    ifunc3 = 'BSpline'

    ofunc1 = 'GradientLine'
    ofunc2 = 'LBFGSB'
    ofunc3 = 'Powell'
    ofunc4 = 'Gradient'



    # get an image from the array input
    fixed_image = sitk.GetImageFromArray(Im_arr1,)
    fixed_image = sitk.Cast(fixed_image, sitk.sitkFloat32)
    moving_image = sitk.GetImageFromArray(Im_arr2)
    moving_image = sitk.Cast(moving_image, sitk.sitkFloat32)

    # Spatial Transform algorithm depend on tfunc_ which are Euler,BSpline,Similarity,Affine
    if trans_type == tfunc_1:
        transform = sitk.CenteredTransformInitializer(fixed_image,
                                                      moving_image,
                                                      sitk.Euler2DTransform(),
                                                      sitk.CenteredTransformInitializerFilter.GEOMETRY)

    elif trans_type == tfunc_2:
        transform = sitk.BSplineTransformInitializer(fixed_image, [fixed_image.GetDimension(), 8])

    elif trans_type == tfunc_3:
        transform = sitk.CenteredTransformInitializer(fixed_image,
                                                      moving_image,
                                                      sitk.Similarity2DTransform(),
                                                      sitk.CenteredTransformInitializerFilter.GEOMETRY)

    elif trans_type == tfunc_4:
        transform = sitk.CenteredTransformInitializer(fixed_image,
                                                      moving_image,
                                                      sitk.AffineTransform(fixed_image.GetDimension()),
                                                      sitk.CenteredTransformInitializerFilter.GEOMETRY)

    # Similarity Metric algorithms depend on mfunc which are mutual information, mean squares, correlation
    registration_method = sitk.ImageRegistrationMethod()

    if metric_type == mfunc1 :
        registration_method.SetMetricAsMattesMutualInformation(numberOfHistogramBins=50)

    elif metric_type == mfunc2 :
        registration_method.SetMetricAsMeanSquares()

    elif metric_type == mfunc3 :
        registration_method.SetMetricAsCorrelation()

    registration_method.SetMetricSamplingStrategy(registration_method.RANDOM)
    registration_method.SetMetricSamplingPercentage(0.01)

    #Interpolator algorithm depends on ifunc which are Linear, Nearest neighbor, BSpline
    if interpolator_type == ifunc1:
        registration_method.SetInterpolator(sitk.sitkLinear)

    elif interpolator_type == ifunc2:
        registration_method.SetInterpolator(sitk.sitkNearestNeighbor)

    elif interpolator_type == ifunc3:
        registration_method.SetInterpolator(sitk.sitkBSpline)

    # Optimizer algorithms depends on ofunc which are Conjugate Gradient Line Search, LBFGSB, Powell, GradientDescen
    if optimizer_type == ofunc1:
        registration_method.SetOptimizerAsConjugateGradientLineSearch(learningRate=1.0, numberOfIterations=70)

    elif optimizer_type == ofunc2:
        registration_method.SetOptimizerAsLBFGSB()

    elif optimizer_type == ofunc3:
        registration_method.SetOptimizerAsPowell()

    elif optimizer_type == ofunc4:
        registration_method.SetOptimizerAsGradientDescent(learningRate=1.0, numberOfIterations=60)

    registration_method.SetOptimizerScalesFromPhysicalShift()

    registration_method.SetInitialTransform(transform)
    registration_method.Execute(fixed_image, moving_image)

    return transform

"""
  When the.py file is run directly, the code block under if __name__ = '__main__' is run.
  When the.py file is imported as a module, the code block under if __name__ = '__main__' is not run.
"""
if __name__ == '__main__':

#input, preprocessing and registration
    path1 = './src_image/CT_Head_Patient1.mhd'
    fixed_im = sitk.ReadImage(path1)
    fixed_image_arr = sitk.GetArrayFromImage(fixed_im)
    path2 = './src_image/MR_Head_Patient1.mhd'
    moving_im = sitk.ReadImage(path2)
    moving_image_arr = sitk.GetArrayFromImage(moving_im)
    transform = itk2DImageRegistration( fixed_image_arr, moving_image_arr,'BSpline','correlation','BSpline','Gradient')

    #resample and Rescale Intensity
    resampler = sitk.ResampleImageFilter()
    resampler.SetTransform(transform)
    resampler.SetReferenceImage(fixed_im)
    out = resampler.Execute(moving_im)
    img0 = sitk.Cast(sitk.RescaleIntensity(moving_im), sitk.sitkUInt8)
    img1 = sitk.Cast(sitk.RescaleIntensity(fixed_im), sitk.sitkUInt8)
    img2 = sitk.Cast(sitk.RescaleIntensity(out), sitk.sitkUInt8)
    img3 = sitk.Cast(sitk.RescaleIntensity(img1 / 2. + img2 / 2.), sitk.sitkUInt8)

    # combine these three scalar images into a multicomponent image named img_out
    img_out = sitk.Compose(img1, img2, img3)
    sitk.WriteImage(img_out,'./src_image/re_Head_Patient1.mhd')



"""
Script Name   : itkRegionGrow.py
Author        : Han
Created       : 2018/5/18
Version       : 1.0
Description   :
  PURPOSE     : This is a function which using the simplest segmentation approches,region growing
  INPUTS      :
  - im_arr    : Im_arr is the array of the image
                Type of data: ndarray
  - type_str  : the type of Region Grow that you want to process
                Type of data: str
                'CT'= 'ConnectedThreshold'
                'CC'= 'ConfidenceConnected'

   OUTPUTS    :
   - im_new   : Image
"""


import SimpleITK as sitk
import itkEdgePreservedSmoothing

def itkRegionGrow(im_arr, type_str, seedlist, lower=0, upper=1):

    func_1 = 'CT'
    func_2 = 'CC'

    # get an image from the array input
    image = itkEdgePreservedSmoothing.itkEdgePreservedSmoothing(im_arr, 'CF')
    image = sitk.GetImageFromArray(im_arr)
    image = sitk.Cast(image, sitk.sitkFloat32)

    # find out the way to process the image according to type_str
    if type_str == func_1:
        im_new = sitk.ConnectedThreshold(image, seedlist, lower, upper)

    elif type_str == func_2:
        im_new = sitk.ConfidenceConnected(image, seedlist)
    
    else:
        print('Please check your spelling,'
              'and try again.')
    return im_new


# an example of using the function
if __name__ == '__main__':

    impath = './src_image/CT159.dcm'
    image = sitk.ReadImage(impath)
    print(image.GetDimension(), image.GetPixelID(), image.GetPixelIDValue(), image.GetSize())
    image_arr = sitk.GetArrayFromImage(image)
    image_new = itkRegionGrow(image_arr, 'CC',[[271, 262, 0]],-50,50)
    print(image_new.GetDimension(), image_new.GetPixelID(), image_new.GetPixelIDValue(), image_new.GetSize())
    image_test = sitk.Cast(image_new, sitk.sitkInt16)
    print(image_test.GetDimension(), image_test.GetPixelID(), image_test.GetPixelIDValue(), image_test.GetSize())
    sitk.WriteImage(image_test, './src_image/yi.dcm')
    sitk.Show(image_test)


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Script Name	  : ReadMhd_WriteOther
Author		  : ZengXiao
Created		  : 2018/4/5
Modified      : 2018/4/7
Version		  : 2.0
Description	  :
PURPOSE：
    Convert Mhd to nii and nrrd format.

INPUTS:
    path: The path of the file which to be converted.
        Type of data: str
    my_format: The format we want to convert to.
        Type of data: str

RETURNED VALUE:
    The converted file named 'result' in the same path.

Modified :
    Add notes and rewritten the nodes as function.

"""
import SimpleITK as sitk


def format_conv(path, my_format):
    # Define function format_conv
    src = sitk.ReadImage(path)                  # Call SimpleITK to read the image
    img_array = sitk.GetArrayFromImage(src)     # Convert image to array
    img = sitk.GetImageFromArray(img_array)     # Convert array to image

    # Get the path the file located
    l = len(path)
    for i in range(0, l):
        if path[l - 1 - i] == '\\':       # Find the last \
            end = l - i
            break

    self_path = path[0:end]              # Get the path of file

    if my_format == 'nii':                                  # chose which format to save
        sitk.WriteImage(img, self_path + 'result.nii')      # save the file named result in the same path
        print('format_conv')                                # Tell the user function worked successfully
    elif my_format == 'nrrd':
        sitk.WriteImage(img, self_path + 'result.nrrd')
        print('format_conv')
    else:
        print('Sorry, We do not support this format.')   # If there is an unexpected format, inform the error


''' Test function '''
path = "D:\PythonCode\zx\src\PATIENT_DICOM.mhd"     # Input path parameters
my_format1 = "nii"              # Input format parameters
format_conv(path, my_format1)   # Call function

my_format2 = "nrrd"             # Try another format
format_conv(path, my_format2)

my_format3 = "txt"              # Try false format
format_conv(path, my_format3)


from PIL import Image

im_path = r'F:\Jupyter Notebook\csv_time_datetime_PIL\rabbit.jpg'
im = Image.open(im_path)
width, height = im.size

print(im.size, width, height)
print(im.format, im.format_description)

im.save(r'C:\Users\Administrator\Desktop\rabbit_copy.jpg')
im.show()
#读取文件

im = Image.open(im_path)
cropedIm = im.crop((700, 100, 1200, 1000))
cropedIm.save(r'C:\Users\Administrator\Desktop\cropped.png')
#裁剪图像
im = Image.open(im_path)
cropedIm = im.crop((700, 100, 1200, 1000))
im.paste(cropedIm, (0, 0))
im.show()
im.save(r'C:\Users\Administrator\Desktop\paste.png')
#复制粘贴


import pylab as y    #引入pylab模块（pylab属于matplotlib的一个模块），并用y代替此模块

x = y.np.linspace(-10, 10, 100)  #设置x横坐标范围和点数，此为numpy中的，返回值为等间距的值
                                #numpy.linspace(a,b,c):a指的是开始位置，b表示的是结束位置，c表示产生点的个数（默认为50）

y.plot(x, x*x*x,'or')  #生成图像，此图像为y=x^3，后面加上‘o'表示为散点图
                        #'r'可设置颜色为红色

ax = y.gca()

ax.spines['right'].set_color('none')
ax.spines['top'].set_color('none')
ax.xaxis.set_ticks_position('bottom')

ax.spines['bottom'].set_position(('data', 0))
ax.yaxis.set_ticks_position('left')

ax.spines['left'].set_position(('data', 0))
ax.set_yticks([-1000, -500, 500, 1000])

y.xlim(x.min() , x.max() ) #将横坐标设置为x的最大值和最小值
y.show() #显示图像


"""
  File name: EdgeDetect_demo
   Function：实现调用多个库的代码,使用边缘检测实现简单的线条画
    File No: 待定
 Created on: 2017-11-2
     Author: zengxiao
Modified on：217-1-14
     Author： zx
Description： 规范注释书写
"""

#-*- coding: utf8 -*-

import numpy as np
from PIL import Image
import matplotlib.pyplot as plt
import xlsxwriter
import xlrd


img=Image.open('C:users/zengxiao/Desktop/ImageProcess/lena.jpg')    #使用PIL打开图片
makebw=np.array(img)                                                      #使用numpy得到像素矩阵
rows,cols,gre=makebw.shape                                                #使用numpy得到矩阵大小

#检测图像边缘

for x in range(rows-1):
    for y in range(cols-1):
        here=makebw[x,y]
        right=makebw[x+1,y]
        above=makebw[x,y+1]
        hereL=((here[0]+here[1]+here[2])/3)
        rightL=((right[0]+right[1]+right[2])/3)
        aboveL=((above[0]+above[1]+above[2])/3)
        if abs(hereL-rightL)>10 and abs(hereL-aboveL)>10:   #设置阈值10，像素变化速度的阈值
            makebw[x,y,0]=0
            makebw[x,y,1]=0
            makebw[x,y,2]=0
        if abs(hereL-rightL)<=10 and abs(hereL-aboveL)<=10:
            makebw[x,y,0]=255
            makebw[x,y,1]=255
            makebw[x,y,2]=255

#使用xlsxwriter将处理后的图像矩阵存入Excel文件中

file  = xlsxwriter.Workbook('C:/Users/zengxiao/Desktop/ImageProcess/LineDrawingMarrey.xlsx')    #创建Excel文件
sheet1 = file.add_worksheet('R')
sheet2 = file.add_worksheet('G')
sheet3 = file.add_worksheet('B')
for x in range(rows):
    for y in range(cols):
        sheet1.write(x, y, makebw[x,y,0])
        sheet2.write(x, y, makebw[x,y,1])
        sheet3.write(x, y, makebw[x,y,2])
file.close()

#使用xlrd从Excel中读取数据

data = xlrd.open_workbook('C:/Users/zengxiao/Desktop/ImageProcess/LineDrawingMarrey.xlsx')
table1 = data.sheets()[0]
table2 = data.sheets()[1]
table3 = data.sheets()[2]
nrows = table1.nrows
ncols = table1.ncols
remakebw=np.zeros((nrows,ncols,3))
for x in range(nrows):
    for y in range(ncols):
        remakebw[x,y,0] = table1.cell(x, y).value
        remakebw[x,y,1] = table2.cell(x, y).value
        remakebw[x,y,2] = table3.cell(x, y).value

#调用matplotible显示图像

makebwimg = Image.fromarray(makebw)
plt.figure('Line drawing')
plt.imshow(makebwimg)
plt.show()                                                                         #显示线条画图片
makebwimg.save('C:/Users/zengxiao/Desktop/ImageProcess/LineDrawing.jpg')    #存储线条画图片





from PIL import Image

im_path = r'C:\Users\lenovo\Desktop\11.jpg'
im = Image.open(im_path)
width, height = im.size

print(im.size, width, height)

print(im.format, im.format_description)
 
im.save(r'C:\Users\lenovo\Desktop\11.jpg')
im.show()

	
